import json
import os
import pandas as pd


def translate_columns(data_folder, df, description_filename):
    description_file_path = os.path.join(data_folder, description_filename)

    with open(description_file_path, "r") as description_file:
        descriptions = json.loads(description_file.read().encode().decode("utf-8-sig"))
    descriptions = {
        column_dict["Nome campo"]: column_dict for column_dict in descriptions
    }
    df.rename(columns=lambda col: descriptions[col]["Field name"], inplace=True)
    return df


def set_time_index(df, drop_hour=True):
    if drop_hour:
        lambda_func = lambda x: x.split(" ")[0]
    else:
        labda_func = lambda x: x

    timestamp = pd.DatetimeIndex(df["date"].apply(lambda_func))
    df.set_index(timestamp, inplace=True)
    del df["date"]
    return df


def prepare_dataframe(
    data_folder, df_filename, description_filename, use_time_index=False
):
    data_file_path = os.path.join(data_folder, df_filename)
    df = pd.read_csv(data_file_path)
    del df["note_it"]
    del df["note_en"]
    df = translate_columns(data_folder, df, description_filename)
    if use_time_index:
        df = set_time_index(df)
    return df


def get_province_structure(df_provinces):
    """Extract the province/region structure from the province dataframe."""

    def get_province_list(region):
        """Get list of provinces for a given region."""
        provinces = set(df_provinces.loc[df_provinces["region"] == region]["province"])
        provinces.discard("In fase di definizione/aggiornamento")
        return list(provinces)

    regions = df_provinces["region"].unique()
    return {region: get_province_list(region) for region in regions}


def get_region_populations():
    import sys
    from SPARQLWrapper import SPARQLWrapper, JSON

    endpoint_url = "https://query.wikidata.org/sparql"

    query = """SELECT DISTINCT ?istatid (MAX(?population) AS ?maxPopulation)
    WHERE
    {
      ?region wdt:P31 wd:Q16110 .
      ?region wdt:P1082 ?population .
      ?region wdt:P635 ?istatid
    }
    GROUP BY ?istatid
    ORDER BY DESC(?maxPopulation)
    """

    def get_results(endpoint_url, query):
        user_agent = "WDQS-example Python/%s.%s" % (
            sys.version_info[0],
            sys.version_info[1],
        )
        # TODO adjust user agent; see https://w.wiki/CX6
        sparql = SPARQLWrapper(endpoint_url, agent=user_agent)
        sparql.setQuery(query)
        sparql.setReturnFormat(JSON)
        return sparql.query().convert()

    results = get_results(endpoint_url, query)

    return {
        int(result["istatid"]["value"]): int(result["maxPopulation"]["value"])
        for result in results["results"]["bindings"]
    }
